#define IR_INPUT PIN_B0         // the IR-Decoder has to be connected to an interrupt pin 

#define IR_STATUS (!input(IR_INPUT))  // invert signal from IR-Receiver 

typedef struct 
{ 
int8 data[2]; 
int8 state; 
} rc5_struct; 

rc5_struct rc5; 

//========================================================================	
// RC5 decoding 
int1 get_RC5(void) 
{ 
   int16 tmp,t; 
   int i; 
   int1 inp; 

   set_timer1(0);                  // reset timer 
   while(IR_STATUS==1);               
   t=get_timer1();                  

   //if ((t<400) || (t<800)) return 0;   // no RC5 code, cancel decoding 

   // receive all 13 data bits 
   for (i=0;i<13;i++) 
   { 

      inp=IR_STATUS;               
      set_timer1(0);               

      while (IR_STATUS==inp)         
      { 
         t=get_timer1();            
         if (t>800) return 0;         // timing wrong -> no RC5 
                              
      } 

      tmp<<=1;                     

      if (inp==0) tmp++;            

      set_timer1(0); 
      while (get_timer1()<776);  
   } 

   tmp=tmp | 0x3000;      // clear Bits 13,14 & 15 

   tmp=tmp & 0x37ff;      // cut off Togglebit 

   rc5.data[0]=tmp & 0xff;  // extract device address 

   tmp>>=8;               

   rc5.data[1]=tmp & 0xff;  // extract command code 

   rc5.state = 1;         // set flag for successful IR command decoding 

   disable_interrupts(INT_EXT);      // disable Interrupts until received command has been processed 

   return 1; 
} 

//========================================================================
//
void IR_Code_Load()
{
	// ADDR1 ===================================
	if( read_eeprom(0) == 0xFF ){ IR_ADDR[1] = 218; }
	else{ IR_ADDR[1] = read_eeprom(0); }

	if( read_eeprom(1) == 0xFF ){ IR_DATA[1] = 55; }
	else{ IR_DATA[1] = read_eeprom(1); }

	// ADDR2 ===================================
	if( read_eeprom(2) == 0xFF ){ IR_ADDR[2] = 213; }
	else{ IR_ADDR[2] = read_eeprom(2); }

	if( read_eeprom(3) == 0xFF ){ IR_DATA[2] = 181; }
	else{ IR_DATA[2] = read_eeprom(3); }

	// ADDR3 ===================================
	if( read_eeprom(4) == 0xFF ){ IR_ADDR[3] = 245; }
	else{ IR_ADDR[3] = read_eeprom(4); }

	if( read_eeprom(5) == 0xFF ){ IR_DATA[3] = 55; }
	else{ IR_DATA[3] = read_eeprom(5); }

	// ADDR4 ===================================
	if( read_eeprom(6) == 0xFF ){ IR_ADDR[4] = 187; }
	else{ IR_ADDR[4] = read_eeprom(6); }

	if( read_eeprom(7) == 0xFF ){ IR_DATA[4] = 55; }
	else{ IR_DATA[4] = read_eeprom(7); }

	// ADDR5 ===================================
	if( read_eeprom(8) == 0xFF ){ IR_ADDR[5] = 171; }
	else{ IR_ADDR[5] = read_eeprom(8); }

	if( read_eeprom(9) == 0xFF ){ IR_DATA[5] = 55; }
	else{ IR_DATA[5] = read_eeprom(9); }

	// ADDR6 ===================================
	if( read_eeprom(10) == 0xFF ){ IR_ADDR[6] = 251; }
	else{ IR_ADDR[6] = read_eeprom(10); }

	if( read_eeprom(11) == 0xFF ){ IR_DATA[6] = 55; }
	else{ IR_DATA[6] = read_eeprom(11); }

}
//========================================================================
//
void Remote_CMD()
{
	      if(rc5.state==1)  // did we receive a valid command? 
	      { 
	
		      	rc5.state = 0;            
		
				LCDGotoXY(1,2);
		      	printf(WriteLCDData,"IR: %u", rc5.data[0]);   		// display device address 
		      	printf(WriteLCDData," IR2: %u", rc5.data[1]);    	// display command code 

				if( rc5.data[0] == IR_ADDR[1] && rc5.data[1] == IR_DATA[1] )
				{  
					output_toggle(OP1);  
					SW1_flag = !SW1_flag; 
				}

				if( rc5.data[0] == IR_ADDR[2] && rc5.data[1] == IR_DATA[2] )
				{  
					output_toggle(OP2);  
					SW2_flag = !SW2_flag; 
				}

				if( rc5.data[0] == IR_ADDR[3] && rc5.data[1] == IR_DATA[3] )
				{  
					output_toggle(OP3);  
					SW3_flag = !SW3_flag; 
				}
	
				if( rc5.data[0] == IR_ADDR[4] && rc5.data[1] == IR_DATA[4] )
				{  
					output_toggle(OP4);  
					SW4_flag = !SW4_flag; 
				}
				if( rc5.data[0] == IR_ADDR[5] && rc5.data[1] == IR_DATA[5] )
				{ 
					output_toggle(OP5); 
					SW5_flag = !SW5_flag; 
				}
				if( rc5.data[0] == IR_ADDR[6] && rc5.data[1] == IR_DATA[6] )
				{ 
					output_toggle(OP6);  
					SW6_flag = !SW6_flag;
				}
	
	        	enable_interrupts(INT_EXT);   // reenable interrupts 
	      } 

}

//========================================================================	
// interrupt function for external interrupt pin 
#int_EXT 
EXT_isr() 
{ 
   output_high(LED2);
   get_RC5(); 
} 
//========================================================================	